/****************************************************************************
* HelpPanel.java
*
* PURPOSE
* A panel displayed on the help window that shows a key of the nodes
* available on an MHealth screen.
*
* MHEALTH
* Written by David Makofske and Kevin Almeroth
* University of California at Santa Barbara
* Department of Computer Science
* Neworking and Multimedia Systems Laboratory
* http://imj.ucsb.edu/mcontrol
*
* COPYRIGHT
* Copyright 1998, 1999
* University of California at Santa Barbara
* Networking and Multimedia Systems Lab
* Santa Barbara, CA  93106
* ALL RIGHTS RESERVED
*
* This material may be modified, copied and redistributed, both within
* the recipient's organization and externally, subject to the following
* restrictions:  (a)  The recipient may not derive income from the
* University of California at Santa Barbara (herein "UCSB") information
* itself;  (b)  In any material based on this information, the recipient
* agrees to acknowledge UCSB; (c)  Any copies made of this material must be
* accompanied by the following copyright notice: "Copyright 1998, 1999
* University of California at Santa Barbara. Santa Barbara, CA 93106. All
* Rights Reserved."; and (d)  The recipient agrees to obey all U.S.
* Government restrictions governing redistribution or export of such
* information. These restrictions may apply to redistribution within an
* international organization. UCSB makes no warranties or representations,
* either expressed or implied, with respect to the results containied
* herein, its quality, merchantability, performance or fitness for a
* particular purpose. In no event shall UCSB or its developers, directors,
* officers, employees or affiliates be liable for direct, incidental,
* indirect, special or consequential damages (including damages or loss
* of business profits, business interruption, loss of business information
* and the like) resulting from any defect in this material or its
* documentation or arising out of the use or inability to use this material
* or accompanying documentation even if UCSB, an authorized representative
* or a UCSB affiliate has been advised of the possibility of such damage.
* UCSB makes no representation or warranty regarding the results obtainable
* through use of this material.  No oral or written information or advice
* given by UCSB, its dealers, distributors, agents, affiliates, developers,
* directors, officers or employees shall create a warranty or in any way
* increase the scope of this warranty.
****************************************************************************/

import java.awt.*;

public
class HelpPanel extends Panel {

  public final int BOX_WIDTH           = 100; // 250
  public final int BOX_HEIGHT          = 20;
  public final int RECEIVER_BOX_HEIGHT = 40;

  public final int VERTICAL_SPACING    = 20;
  public final int HORIZONTAL_SPACING  = 5;

  private int fontSize = 10;

  // draw to offscreen graphics from example GraphLayout (SUN) 
  Image offscreen = null;
  Dimension offscreensize = null;
  Dimension preferred_size = null;
  Graphics offgraphics = null;

  // some layout constants
  public Color normal_line_color    = Color.black;
  public Color congested_line_color = Color.red;
  public Color background_color     = Color.white;

  public HelpPanel() {
    preferred_size = new Dimension(920,100);
    setBackground(Color.white);
  }

  public Dimension getPreferredSize() { 
    return preferred_size;
  }
  
  public void paint(Graphics g) {
    //Debug.out(debugLevel, Debug.INFO, "TreePanel: paint()");
    int x,y;
    int yTop;
    FontMetrics fm;
    int font_width, font_height;
    int drop=BOX_HEIGHT;
    String text;
    int horizontal_spacing=30;

    x=10;
    y=20;
    
    Font f = g.getFont();
    g.setFont(new Font(f.getName(), f.getStyle(), 18));
    fm = g.getFontMetrics(g.getFont());
    text = "MHealth Key:";
    g.drawString(text, x,y);
    font_height = fm.getHeight();
    y += font_height;
    yTop = y;

    g.setFont(new Font(f.getName(), f.getStyle(), fontSize));
    fm = g.getFontMetrics(g.getFont());
    font_height = fm.getHeight();

    g.setColor(Color.white);
    g.fillRect(x,y,BOX_WIDTH,BOX_HEIGHT);
    g.setColor(Color.black);
    g.drawRect(x,y,BOX_WIDTH,BOX_HEIGHT);
    text = "128.111.52.15";
    font_width = fm.stringWidth(text);
    y += (BOX_HEIGHT-font_height)/2 + font_height;
    g.drawString(text, x + ((BOX_WIDTH-font_width)/2), y);
    text = "Router";
    font_width = fm.stringWidth(text);
    y += drop;
    g.drawString(text, x + ((BOX_WIDTH-font_width)/2), y);
    text = "(Uncongested)";
    font_width = fm.stringWidth(text);
    y += font_height;
    g.drawString(text, x + ((BOX_WIDTH-font_width)/2), y);

    x += BOX_WIDTH + horizontal_spacing;
    y = yTop;

    g.setColor(Color.green);
    g.fillRect(x,y,BOX_WIDTH,BOX_HEIGHT);
    g.setColor(Color.black);
    g.drawRect(x,y,BOX_WIDTH,BOX_HEIGHT);
    text = "128.111.52.15";
    font_width = fm.stringWidth(text);
    y += (BOX_HEIGHT-font_height)/2 + font_height;
    g.drawString(text, x + ((BOX_WIDTH-font_width)/2), y);
    text = "Sender/Receiver";
    font_width = fm.stringWidth(text);
    y += drop;
    g.drawString(text, x + ((BOX_WIDTH-font_width)/2), y);
    text = "(Uncongested)";
    font_width = fm.stringWidth(text);
    y += font_height;
    g.drawString(text, x + ((BOX_WIDTH-font_width)/2), y);

    x += BOX_WIDTH + horizontal_spacing;
    y = yTop;

    g.setColor(Color.yellow);
    g.fillRect(x,y,BOX_WIDTH,BOX_HEIGHT);
    g.setColor(Color.black);
    g.drawRect(x,y,BOX_WIDTH,BOX_HEIGHT);
    text = "128.111.52.15";
    font_width = fm.stringWidth(text);
    y += (BOX_HEIGHT-font_height)/2 + font_height;
    g.drawString(text, x + ((BOX_WIDTH-font_width)/2), y);
    text = "Router/Sender/Receiver";
    font_width = fm.stringWidth(text);
    y += drop;
    g.drawString(text, x + ((BOX_WIDTH-font_width)/2), y);
    text = "(2%-9% Loss)";
    font_width = fm.stringWidth(text);
    y += font_height;
    g.drawString(text, x + ((BOX_WIDTH-font_width)/2), y);
    
    x += BOX_WIDTH + horizontal_spacing;
    y = yTop;

    g.setColor(Color.red);
    g.fillRect(x,y,BOX_WIDTH,BOX_HEIGHT);
    g.setColor(Color.black);
    g.drawRect(x,y,BOX_WIDTH,BOX_HEIGHT);
    text = "128.111.52.15";
    font_width = fm.stringWidth(text);
    y += (BOX_HEIGHT-font_height)/2 + font_height;
    g.drawString(text, x + ((BOX_WIDTH-font_width)/2), y);
    text = "Router/Sender/Receiver";
    font_width = fm.stringWidth(text);
    y += drop;
    g.drawString(text, x + ((BOX_WIDTH-font_width)/2), y);
    text = "(10% + Loss)";
    font_width = fm.stringWidth(text);
    y += font_height;
    g.drawString(text, x + ((BOX_WIDTH-font_width)/2), y);
   
    x += BOX_WIDTH + horizontal_spacing;
    y = yTop;

    g.setColor(Color.pink);
    g.fillRect(x,y,BOX_WIDTH,BOX_HEIGHT);
    g.setColor(Color.black);
    g.drawRect(x,y,BOX_WIDTH,BOX_HEIGHT);
    text = "128.111.52.15";
    font_width = fm.stringWidth(text);
    y += (BOX_HEIGHT-font_height)/2 + font_height;
    g.drawString(text, x + ((BOX_WIDTH-font_width)/2), y);
    text = "Sender/Receiver";
    font_width = fm.stringWidth(text);
    y += drop;
    g.drawString(text, x + ((BOX_WIDTH-font_width)/2), y);
    text = "(Unknown Loss)";
    font_width = fm.stringWidth(text);
    y += font_height;
    g.drawString(text, x + ((BOX_WIDTH-font_width)/2), y);

    x += BOX_WIDTH + horizontal_spacing;
    y = yTop;

    g.setColor(Color.white);
    g.fillRect(x,y,BOX_WIDTH,BOX_HEIGHT);
    g.setColor(Color.green);
    g.fillRect(x,y+BOX_HEIGHT-3,BOX_WIDTH,3);
    g.setColor(Color.black);
    g.drawRect(x,y,BOX_WIDTH,BOX_HEIGHT);
    text = "128.111.52.15";
    font_width = fm.stringWidth(text);
    y += (BOX_HEIGHT-font_height)/2 + font_height;
    g.drawString(text, x + ((BOX_WIDTH-font_width)/2), y);
    text = "Router w/";
    font_width = fm.stringWidth(text);
    y += drop;
    g.drawString(text, x + ((BOX_WIDTH-font_width)/2), y);
    text = "Pruned Nodes";
    font_width = fm.stringWidth(text);
    y += font_height;
    g.drawString(text, x + ((BOX_WIDTH-font_width)/2), y);

    x += BOX_WIDTH + horizontal_spacing;
    y = yTop;

    g.setColor(Color.lightGray);
    g.fillRect(x,y,BOX_WIDTH,BOX_HEIGHT);
    g.setColor(Color.black);
    g.drawRect(x,y,BOX_WIDTH,BOX_HEIGHT);
    text = "128.111.52.15";
    font_width = fm.stringWidth(text);
    y += (BOX_HEIGHT-font_height)/2 + font_height;
    g.drawString(text, x + ((BOX_WIDTH-font_width)/2), y);
    text = "Sender/Receiver";
    font_width = fm.stringWidth(text);
    y += drop;
    g.drawString(text, x + ((BOX_WIDTH-font_width)/2), y);
    text = "About to Time Out";
    font_width = fm.stringWidth(text);
    y += font_height;
    g.drawString(text, x + ((BOX_WIDTH-font_width)/2), y);

    x += BOX_WIDTH + horizontal_spacing;
    y += font_height;

    preferred_size = new Dimension(x,y);
//    System.out.println(x + " " + y);
  }
}
