/****************************************************************************
* Menu.java
*
* PURPOSE
* MHealth menu of options.
*
* MHEALTH
* Written by David Makofske and Kevin Almeroth
* University of California at Santa Barbara
* Department of Computer Science
* Neworking and Multimedia Systems Laboratory
* http://imj.ucsb.edu/mcontrol
*
* COPYRIGHT
* Copyright 1998, 1999
* University of California at Santa Barbara
* Networking and Multimedia Systems Lab
* Santa Barbara, CA  93106
* ALL RIGHTS RESERVED
*
* This material may be modified, copied and redistributed, both within
* the recipient's organization and externally, subject to the following
* restrictions:  (a)  The recipient may not derive income from the
* University of California at Santa Barbara (herein "UCSB") information
* itself;  (b)  In any material based on this information, the recipient
* agrees to acknowledge UCSB; (c)  Any copies made of this material must be
* accompanied by the following copyright notice: "Copyright 1998, 1999
* University of California at Santa Barbara. Santa Barbara, CA 93106. All
* Rights Reserved."; and (d)  The recipient agrees to obey all U.S.
* Government restrictions governing redistribution or export of such
* information. These restrictions may apply to redistribution within an
* international organization. UCSB makes no warranties or representations,
* either expressed or implied, with respect to the results containied
* herein, its quality, merchantability, performance or fitness for a
* particular purpose. In no event shall UCSB or its developers, directors,
* officers, employees or affiliates be liable for direct, incidental,
* indirect, special or consequential damages (including damages or loss
* of business profits, business interruption, loss of business information
* and the like) resulting from any defect in this material or its
* documentation or arising out of the use or inability to use this material
* or accompanying documentation even if UCSB, an authorized representative
* or a UCSB affiliate has been advised of the possibility of such damage.
* UCSB makes no representation or warranty regarding the results obtainable
* through use of this material.  No oral or written information or advice
* given by UCSB, its dealers, distributors, agents, affiliates, developers,
* directors, officers or employees shall create a warranty or in any way
* increase the scope of this warranty.
****************************************************************************/

import java.awt.*;
import java.io.*;
import java.awt.event.*;
import java.util.*;

public class Menu extends Dialog implements ItemListener {

  private MainWindow parent;
  private SessionData sessionData;

  private BorderedPanel pnlAddress;
  private TextField tfMulticastPort, tfMulticastIP;

  private BorderedPanel pnlLogging;
  private Checkbox chkRtcpLogging, chkRouteLogging;
  private Label lblLogDir, lblLogFileName;
  private TextField tfLogDir;
  private TextField tfLogFileName;
  
  private BorderedPanel pnlImageDump;
  private Checkbox chkImageDump;
  private Label lblImageDumpDir, lblImageDumpFreq;
  private TextField tfImageDumpDir, tfImageDumpFreq;

  private Label lblStatus;

  private Panel pnlButtons;
  private Button btnApply, btnDismiss;
 
  private Vector listener=null;

  private final static int SESSION_ADDRESS_CHANGED       = 0;
  private final static int LOG_FILENAME_DIR_CHANGED      = 1;
  private final static int RTCP_LOGGING_CHANGED          = 2;
  private final static int ROUTE_LOGGING_CHANGED         = 3;

  public Menu(MainWindow parent, MHealth mhealth, SessionData sessionData) {
    super(parent, "MHealth Menu", true);

    this.parent = parent;
    this.sessionData = sessionData;

    //setSize(500,280);

    addWindowListener(new WindowAdapter() {
                        public void windowClosing(WindowEvent event) {
                          Dismiss();
                        }
                      });

    setLayout(new FlowLayout());

    GridBagLayout layout = new GridBagLayout();
    setLayout(layout);

    GridBagConstraints gbc = new GridBagConstraints();
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.weightx = 0;
    gbc.weighty = 0;

    /*******************************************************************
     ********************** Session Info Panel *************************
     *******************************************************************/
    pnlAddress = new BorderedPanel();

    Label lbl = new Label("Session Info", Label.CENTER);
    Font headingFont = new Font("TimesRoman",Font.BOLD, 18);
    lbl.setFont(headingFont);
    pnlAddress.addToGridBag(lbl,0,0,2,1);

    pnlAddress.addToGridBag(new Label("Multicast IP Address:",Label.RIGHT), 
                            0, 1, 1, 1);
/*    MCIP = new String("224.2.127.2");
    MCIP = new String("224.2.186.225"); // KMGE Magic
    MCIP = new String("224.2.0.1"); // MBone Audio
    MCIP = new String("224.2.246.107"); // KVRM Alternative
    MCIP = new String("224.2.168.1"); // Capitol Connection GMU
    MCIP = new String("224.2.253.119"); // Radio Free Vat
    MCIP = new String("224.2.229.11"); // DARPA/ISO
    MCIP = new String("224.2.185.163"); // RISQ Radio*/

    tfMulticastIP = new TextField(sessionData.getSessionIP(),20);
    pnlAddress.addToGridBag(tfMulticastIP, 1, 1, 1, 1);

    pnlAddress.addToGridBag(new Label("Video Port:",Label.RIGHT), 0, 2, 1, 1);

/*    MCport = new String("4640");
    MCport = new String("26406"); // KMGE Magic 
    MCport = new String("23456"); // MBone Audio
    MCport = new String("22130"); // KVRM Alternative
    MCport = new String("28768"); // Capitol Connection GMU
    MCport = new String("42418"); // Radio Free Vat
    MCport = new String("57690"); // DARPA/ISO
    MCport = new String("31202"); // RISQ Radio */

    tfMulticastPort = new TextField(sessionData.getSessionPort(),6);
    pnlAddress.addToGridBag(tfMulticastPort, 1, 2, 1, 1);

    addToGridBag(pnlAddress, gbc, 0, 0, 1, 1);

    /*******************************************************************
     ********************** Logging Panel ******************************
     *******************************************************************/
    pnlLogging = new BorderedPanel();
    pnlLogging.setFill("VERTICAL");

    lbl = new Label("Logging", Label.CENTER);
    lbl.setFont(headingFont);
    pnlLogging.addToGridBag(lbl,0,0,2,1);

    chkRtcpLogging = new Checkbox("Enable Rtcp Packet Logging", 
                                  sessionData.getRtcpLoggingEnabled());
    chkRtcpLogging.addItemListener(this);
    pnlLogging.addToGridBag(chkRtcpLogging, 0, 1, 2, 1);

    chkRouteLogging = new Checkbox("Enable Route Logging",  
                                   sessionData.getRouteLoggingEnabled());
    chkRouteLogging.addItemListener(this);
    pnlLogging.addToGridBag(chkRouteLogging, 0, 2, 2, 1);

    lblLogDir = new Label("Log Directory:");
    pnlLogging.addToGridBag(lblLogDir, 0, 3, 1, 1);
    tfLogDir = new TextField(sessionData.getLogDir(),15);
    tfLogDir.setEnabled((sessionData.getRouteLoggingEnabled()) ||
                        (sessionData.getRtcpLoggingEnabled()));
    lblLogDir.setEnabled((sessionData.getRouteLoggingEnabled()) ||
                         (sessionData.getRtcpLoggingEnabled()));
    pnlLogging.addToGridBag(tfLogDir, 1, 3, 1, 1);

    lblLogFileName = new Label("Log File Name:");
    pnlLogging.addToGridBag(lblLogFileName, 0, 4, 1, 1);
    tfLogFileName = new TextField(sessionData.getLogFileName(),15);
    tfLogFileName.setEnabled((sessionData.getRouteLoggingEnabled()) ||
                             (sessionData.getRtcpLoggingEnabled()));
    lblLogFileName.setEnabled((sessionData.getRouteLoggingEnabled()) ||
                              (sessionData.getRtcpLoggingEnabled()));
    pnlLogging.addToGridBag(tfLogFileName, 1, 4, 1, 1);

    addToGridBag(pnlLogging, gbc, 0, 1, 1, 1);

    /*******************************************************************
     ********************** Image Dump Panel ***************************
     *******************************************************************/
    pnlImageDump = new BorderedPanel();
    pnlImageDump.setFill("VERTICAL");

    lbl = new Label("Image Dump", Label.CENTER);
    lbl.setFont(headingFont);
    pnlImageDump.addToGridBag(lbl,0,0,2,1);

    chkImageDump = new Checkbox("Create Image Dump",
                                  sessionData.getImageDumpEnabled());
    chkImageDump.addItemListener(this);
    pnlImageDump.addToGridBag(chkImageDump, 0, 1, 2, 1);

    lblImageDumpDir = new Label("Image Dump Directory:");
    pnlImageDump.addToGridBag(lblImageDumpDir, 0, 2, 1, 1);
    tfImageDumpDir = new TextField(sessionData.getImageDumpDir(),15);
    tfImageDumpDir.setEnabled(sessionData.getImageDumpEnabled());
    lblImageDumpDir.setEnabled(sessionData.getImageDumpEnabled());
    pnlImageDump.addToGridBag(tfImageDumpDir, 1, 2, 1, 1);

    lblImageDumpFreq = new Label("Image Dump Frequency:");
    pnlImageDump.addToGridBag(lblImageDumpFreq, 0, 3, 1, 1);
    tfImageDumpFreq = new TextField(sessionData.getImageDumpFreq(),15);
    tfImageDumpFreq.setEnabled(sessionData.getImageDumpEnabled());
    lblImageDumpFreq.setEnabled(sessionData.getImageDumpEnabled());
    pnlImageDump.addToGridBag(tfImageDumpFreq, 1, 3, 1, 1);

    addToGridBag(pnlImageDump, gbc, 0, 2, 1, 1);

    lblStatus = new Label("",Label.CENTER);
    addToGridBag(lblStatus, gbc, 0, 6, 1, 1);
    lblStatus.setForeground(Color.red);

    pnlButtons = new Panel();
    btnApply = new Button("Apply");
    btnApply.addActionListener(
             new ActionListener() {
               public void actionPerformed(ActionEvent evt) {
                 if (Validate()) {
                   Apply();
                   setVisible(false);
                 }
               } 
             });
    pnlButtons.add(btnApply);

    btnDismiss = new Button("Dismiss");
    btnDismiss.addActionListener(
               new ActionListener() {
                 public void actionPerformed(ActionEvent evt) {
                   Dismiss();
                 } 
               });
    pnlButtons.add(btnDismiss);

    addToGridBag(pnlButtons, gbc, 0, 7, 1, 1);

    addWindowListener(new WindowAdapter() { public void
                          windowClosing(WindowEvent e) { setVisible(false);
                          } } );
  
    pack();
    setResizable(false);

    // Center the Menu window on the MHealth main window
    Dimension menu_dimension = getSize();
    Dimension parent_dimension = parent.getSize();
    Point parent_point = parent.getLocation();

    setLocation(new Point(parent_point.x +
                        ((parent_dimension.width - menu_dimension.width)/2),
                        parent_point.y +
                        ((parent_dimension.height - menu_dimension.height)/2)));
    

    listener = new Vector();
    addSessionListener(mhealth);

    setVisible(true);

  }

  public boolean Validate() {
    ValidAddress validaddress = new ValidAddress();
    if (! validaddress.isValidMulticastAddressPort(tfMulticastIP.getText(),
                                             tfMulticastPort.getText())) {
      lblStatus.setText("Multicast Address/Port is not valid.");
      Toolkit.getDefaultToolkit().beep();
      return(false);
    }
//    if (chkRtcpLogging.getState() || chkRouteLogging.getState()) {
      try {
        File f = new File(tfLogDir.getText());
      if (! f.isDirectory()) {
        lblStatus.setText("Log Directory is not valid.");
        Toolkit.getDefaultToolkit().beep();
        return(false);
      }
      } catch (NullPointerException npe) {
        lblStatus.setText("Log Directory is not valid.");
        Toolkit.getDefaultToolkit().beep();
        return(false);
      }
//    }
    if (tfLogFileName.getText() == "") {
      lblStatus.setText("Log File Name is not valid.");
      Toolkit.getDefaultToolkit().beep();
      return(false);
    }
    try {
      File f = new File(tfImageDumpDir.getText());
      if (! f.isDirectory()) {
        lblStatus.setText("Image Dump Directory is not valid.");
        Toolkit.getDefaultToolkit().beep();
        return(false);
      }
    } catch (NullPointerException npe) {
      lblStatus.setText("Image Dump Directory is not valid.");
      Toolkit.getDefaultToolkit().beep();
      return(false);
    } 
    int min;
    try {
      min = Integer.parseInt(tfImageDumpFreq.getText());
      if ((min < 30) || (min > 999)) {
        lblStatus.setText("Image Dump Frequency valid range is 30 to 999 seconds.");
        Toolkit.getDefaultToolkit().beep();
        return(false);
      }
    } catch (NumberFormatException nfe) {
      lblStatus.setText("Image Dump Frequency must be an integer number.");
      Toolkit.getDefaultToolkit().beep();
      return(false);
    }
    return(true);
  }

  public void Apply() {
    if (chkRtcpLogging.getState() != sessionData.getRtcpLoggingEnabled()) {
      sessionData.setRtcpLoggingEnabled(chkRtcpLogging.getState());
      eventNotify(RTCP_LOGGING_CHANGED);
    }

    if (chkRouteLogging.getState() != sessionData.getRouteLoggingEnabled()) {
      sessionData.setRouteLoggingEnabled(chkRouteLogging.getState());
      eventNotify(ROUTE_LOGGING_CHANGED);
    }

    if ((! tfLogDir.getText().equals(sessionData.getLogDir())) ||
        (! tfLogFileName.getText().equals(sessionData.getLogFileName()))) {
      sessionData.setLogDir(tfLogDir.getText());
      sessionData.setLogFileName(tfLogFileName.getText());
      eventNotify(LOG_FILENAME_DIR_CHANGED);
    }

    if ( (! tfMulticastIP.getText().equals(sessionData.getSessionIP())) ||
        (! tfMulticastPort.getText().equals(sessionData.getSessionPort()))) {

      sessionData.setSessionIP(tfMulticastIP.getText());
      sessionData.setSessionPort(tfMulticastPort.getText());
      eventNotify(SESSION_ADDRESS_CHANGED);
    }

    if (chkImageDump.getState() !=  sessionData.getImageDumpEnabled()) {
      sessionData.setImageDumpEnabled(chkImageDump.getState());
    } 
    if (! tfImageDumpDir.getText().equals(sessionData.getImageDumpDir())) {
      sessionData.setImageDumpDir(tfImageDumpDir.getText());
    } 
    if (! tfImageDumpFreq.getText().equals(sessionData.getImageDumpFreq())) {
      sessionData.setImageDumpFreq(tfImageDumpFreq.getText());
    } 
  }

  public void Dismiss() {
    setVisible(false);
  }

  public void addSessionListener(SessionListener l) {
    listener.addElement(l);
  }

  public void removeSessionListener(SessionListener l) {
    listener.removeElement(l);
  }

  private void eventNotify(int Event) {
//    System.out.println("Menu: eventNotify(" + Event + ")");

    if (listener == null) return;     // no one to notify!
    if (listener.size() == 0) return; // no one to notify!

    switch(Event) {
    case SESSION_ADDRESS_CHANGED:
      for (Enumeration e = listener.elements(); e.hasMoreElements(); ) {
        SessionListener sl = (SessionListener) e.nextElement();
        sl.addressChanged(tfMulticastIP.getText(), tfMulticastPort.getText());
      }
      break;
    case LOG_FILENAME_DIR_CHANGED:
      for (Enumeration e = listener.elements(); e.hasMoreElements(); ) {
        SessionListener sl = (SessionListener) e.nextElement();
        sl.logFileNameDirChanged();
      }
      break;
    case RTCP_LOGGING_CHANGED:
      for (Enumeration e = listener.elements(); e.hasMoreElements(); ) {
        SessionListener sl = (SessionListener) e.nextElement();
        sl.rtcpLoggingChanged(chkRtcpLogging.getState());
      }
      break;
    case ROUTE_LOGGING_CHANGED:
      for (Enumeration e = listener.elements(); e.hasMoreElements(); ) {
        SessionListener sl = (SessionListener) e.nextElement();
        sl.routeLoggingChanged(chkRouteLogging.getState());
      }
      break;
    default:
      System.out.println("Menu: unknown event " + Event);
      break;
    }
  }

  public void itemStateChanged(ItemEvent e) {
    Object origin = e.getSource();

    if ((origin == chkRtcpLogging) || (origin == chkRouteLogging)) {
      if ((chkRtcpLogging.getState()) || (chkRouteLogging.getState())) 
        setLoggingOn();
      else
        setLoggingOff();
    } else if (origin == chkImageDump) {
      if (chkImageDump.getState())
        setImageDumpOn();
      else
        setImageDumpOff();
    }
  }

  public void setLoggingOn() {
    tfLogDir.setEnabled(true);
    lblLogDir.setEnabled(true);
    tfLogFileName.setEnabled(true);
    lblLogFileName.setEnabled(true);
  }

  public void setLoggingOff() {
    tfLogDir.setEnabled(false);
    lblLogDir.setEnabled(false);
    tfLogFileName.setEnabled(false);
    lblLogFileName.setEnabled(false);
  }

  public void setImageDumpOn() {
    tfImageDumpDir.setEnabled(true);
    lblImageDumpDir.setEnabled(true);
    tfImageDumpFreq.setEnabled(true);
    lblImageDumpFreq.setEnabled(true);
  }

  public void setImageDumpOff() {
    tfImageDumpDir.setEnabled(false);
    lblImageDumpDir.setEnabled(false);
    tfImageDumpFreq.setEnabled(false);
    lblImageDumpFreq.setEnabled(false);
  }

  public void addToGridBag(Component c, GridBagConstraints gbc,
                           int x, int y, int w, int h) {
    gbc.gridx = x;
    gbc.gridy = y;
    gbc.gridwidth = w;
    gbc.gridheight = h;
    add(c, gbc);
  }
}
