/****************************************************************************
* SourceList.java
*
* PURPOSE
* Class to hold list of multicast sources.
*
* MHEALTH
* Written by David Makofske and Kevin Almeroth
* University of California at Santa Barbara
* Department of Computer Science
* Neworking and Multimedia Systems Laboratory
* http://imj.ucsb.edu/mcontrol
*
* COPYRIGHT
* Copyright 1998, 1999
* University of California at Santa Barbara
* Networking and Multimedia Systems Lab
* Santa Barbara, CA  93106
* ALL RIGHTS RESERVED
*
* This material may be modified, copied and redistributed, both within
* the recipient's organization and externally, subject to the following
* restrictions:  (a)  The recipient may not derive income from the
* University of California at Santa Barbara (herein "UCSB") information
* itself;  (b)  In any material based on this information, the recipient
* agrees to acknowledge UCSB; (c)  Any copies made of this material must be
* accompanied by the following copyright notice: "Copyright 1998, 1999
* University of California at Santa Barbara. Santa Barbara, CA 93106. All
* Rights Reserved."; and (d)  The recipient agrees to obey all U.S.
* Government restrictions governing redistribution or export of such
* information. These restrictions may apply to redistribution within an
* international organization. UCSB makes no warranties or representations,
* either expressed or implied, with respect to the results containied
* herein, its quality, merchantability, performance or fitness for a
* particular purpose. In no event shall UCSB or its developers, directors,
* officers, employees or affiliates be liable for direct, incidental,
* indirect, special or consequential damages (including damages or loss
* of business profits, business interruption, loss of business information
* and the like) resulting from any defect in this material or its
* documentation or arising out of the use or inability to use this material
* or accompanying documentation even if UCSB, an authorized representative
* or a UCSB affiliate has been advised of the possibility of such damage.
* UCSB makes no representation or warranty regarding the results obtainable
* through use of this material.  No oral or written information or advice
* given by UCSB, its dealers, distributors, agents, affiliates, developers,
* directors, officers or employees shall create a warranty or in any way
* increase the scope of this warranty.
****************************************************************************/

import java.util.*;
import java.io.*;

public class SourceList {

  private SourceNode root=null;

  private SourceNode currentSource=null;

  public SourceList() {
    root = null;
  }

  public void addSourceNode(RtcpReport report) {
    if (root == null) {
      SourceNode sn = new SourceNode(report);
      root=sn;
      currentSource=sn;
    } else {
      SourceNode sn = new SourceNode(report);
      SourceNode tail = getSourceTail(getSourceRoot());

      if (tail == null) {
        // List is empty, make this the new root
        root = new SourceNode(report);
      } else {
        // Append to tail of list
        SourceNode newNode = new SourceNode(report);
        newNode.setPrevNode(tail);
        tail.setNextNode(newNode);
      }
    }
  }

  public void removeSourceNode(RtcpReport report) {
    SourceNode sn;

    sn = findSourceNode(getSourceRoot(), report.getIP());

    if (sn == null) return;

    if (sn == getSourceRoot()) {
      // remove root of list
      if (sn.getNextNode() == null) {
        // root is only member
        root=null;
      } else {
        sn.getNextNode().setPrevNode(null);
        root = sn.getNextNode();
      } 
    } else {
      if (sn.getNextNode() == null) {
        sn.getPrevNode().setNextNode(null);
      } else {
        sn.getPrevNode().setNextNode(sn.getNextNode());
        sn.getNextNode().setPrevNode(sn.getPrevNode());
      }
    }
  }

  public SourceNode getSourceRoot() { return root; }

  public boolean isSourceRoot(SourceNode sn) { 
    if (root == null) 
      return false;

    return(sn.getIP().equals(root.getIP()));
  }

  public void timeoutSourceInList(String IP) {
    SourceNode sn;

    sn = findSourceNode(getSourceRoot(), IP);

    if (sn != null) {
      sn.setTimeout(true);
    }
  }


  public SourceNode findSourceNode(SourceNode start, String IP) {
    SourceNode next;

    if (start == null) 
      return(null); 
    else if (IP.equals(start.getIP())) 
      return(start);
    else
      return(findSourceNode(start.getNextNode(), IP));
  }

  public SourceNode getSourceTail(SourceNode start) { 
    if (start == null) 
      return(null);
    else if (start.getNextNode() == null)
      return(start);
    else
      return(getSourceTail(start.getNextNode()));
  }

  public SourceNode findSourceInside(SourceNode start, 
                                         int mouseX, int mouseY) {
    
    if (start == null) return(null);

    if (start.isInside(mouseX, mouseY)) return(start); 

    return(findSourceInside(start.getNextNode(), mouseX, mouseY));
  }

  public boolean isEmpty() {
    return(root == null);
  }

  public void delete() {
    while (root != null) {
      removeSourceNode(root.getRtcpReport());
    }
  }
}
