/****************************************************************************
* TreePanel.java
*
* PURPOSE
* Panel to display an MHealth tree.
*
* MHEALTH
* Written by David Makofske and Kevin Almeroth
* University of California at Santa Barbara
* Department of Computer Science
* Neworking and Multimedia Systems Laboratory
* http://imj.ucsb.edu/mcontrol
*
* COPYRIGHT
* Copyright 1998, 1999
* University of California at Santa Barbara
* Networking and Multimedia Systems Lab
* Santa Barbara, CA  93106
* ALL RIGHTS RESERVED
*
* This material may be modified, copied and redistributed, both within
* the recipient's organization and externally, subject to the following
* restrictions:  (a)  The recipient may not derive income from the
* University of California at Santa Barbara (herein "UCSB") information
* itself;  (b)  In any material based on this information, the recipient
* agrees to acknowledge UCSB; (c)  Any copies made of this material must be
* accompanied by the following copyright notice: "Copyright 1998, 1999
* University of California at Santa Barbara. Santa Barbara, CA 93106. All
* Rights Reserved."; and (d)  The recipient agrees to obey all U.S.
* Government restrictions governing redistribution or export of such
* information. These restrictions may apply to redistribution within an
* international organization. UCSB makes no warranties or representations,
* either expressed or implied, with respect to the results containied
* herein, its quality, merchantability, performance or fitness for a
* particular purpose. In no event shall UCSB or its developers, directors,
* officers, employees or affiliates be liable for direct, incidental,
* indirect, special or consequential damages (including damages or loss
* of business profits, business interruption, loss of business information
* and the like) resulting from any defect in this material or its
* documentation or arising out of the use or inability to use this material
* or accompanying documentation even if UCSB, an authorized representative
* or a UCSB affiliate has been advised of the possibility of such damage.
* UCSB makes no representation or warranty regarding the results obtainable
* through use of this material.  No oral or written information or advice
* given by UCSB, its dealers, distributors, agents, affiliates, developers,
* directors, officers or employees shall create a warranty or in any way
* increase the scope of this warranty.
****************************************************************************/

import java.awt.*;
import java.util.*;
import java.io.*;

public
class TreePanel extends Panel {

  private SessionData sessionData;
  ScrollInterface scroller = null; 
  Dimension preferred_size = null;
  Dimension tree_size = null; 
  private int columns[];
 
  long magicNumber=0;
  private PopupMenuEvent popupmenuevent;

  public final int MAX_HOPS            = 40;

  public final int BOX_WIDTH           = 100; // 250
  public final int BOX_HEIGHT          = 20;
  public final int RECEIVER_BOX_HEIGHT = 40;

  public final int VERTICAL_SPACING    = 20;
  public final int HORIZONTAL_SPACING  = 5;

  private int fontSize = 10;

  // draw to offscreen graphics from example GraphLayout (SUN) 
  Image offscreen = null;
  Dimension offscreensize = null;
  Graphics offgraphics = null;

  // some layout constants
  public Color normal_line_color    = Color.black;
  public Color congested_line_color = Color.red;
  public Color background_color     = Color.white;

  public TreePanel(ScrollInterface scroller, SessionData sessionData, 
                   int width, int height) {
    this.scroller = scroller;
    this.sessionData = sessionData;
    preferred_size = new Dimension(width,height);
    tree_size = new Dimension(0,0);

    popupmenuevent = new PopupMenuEvent(this, scroller, sessionData.tree, 
                                        sessionData.receiverList,
                                        sessionData.sourceList);
  }
  
  public Dimension getPreferredSize() { 
    return(preferred_size);
  }
  
  public void redraw() {
    //System.out.println("TreePanel: redraw()");

    if (sessionData.tree != null)
      computeAll(); 

    magicNumber = new Date().getTime();
    repaint();
  }
  
  public void computeAll() {
    Dimension extent;

    //System.out.println("TreePanel: computeAll()");

    columns = new int[MAX_HOPS];
    if (sessionData.tree.getRoot() != null) 
      extent = computeTree(sessionData.tree.getRoot());
    else
      extent = new Dimension(0,0);
    
    int x = HORIZONTAL_SPACING;
    int y = getSize().height - VERTICAL_SPACING - RECEIVER_BOX_HEIGHT;
    if (extent.height > y) 
      y = extent.height + VERTICAL_SPACING;
    int width = computeReceivers(sessionData.receiverList.getReceiverRoot(),x,y,
                                  extent.width);
    if (width > extent.width)
      tree_size.width = width;
    else
      tree_size.width = extent.width;
    y += RECEIVER_BOX_HEIGHT + VERTICAL_SPACING;
    if (y > extent.height) 
      tree_size.height = y;
    else
      tree_size.height = extent.height;

    x = HORIZONTAL_SPACING;
    y = VERTICAL_SPACING;
    width = computeSources(sessionData.sourceList.getSourceRoot(),x,y, 
                           extent.width);

    //System.out.println("TreePanel: tree computed, tree_size is "+ tree_size);
  }

  public Dimension computeTree(TreeNode node) {
    int max_height=0, max_width=0;

    //System.out.print("TreePanel: computeTree(");
    //if (node != null) System.out.print(node.name+"/"+node.IP+"/level "+
    //                      node.level);
    //System.out.println(")");
              
/*    System.out.print("coldump: ");
    for (int x=0; x < 15; x++) {
      System.out.print(x + "=" + columns[x]+"/");
    }
    System.out.println();*/

    Dimension child_extent = null;
    if (node.magicNumber != magicNumber) {
      node.setX((columns[node.level]*BOX_WIDTH) +
                ((columns[node.level]+1)*HORIZONTAL_SPACING));
      node.setY((node.level*BOX_HEIGHT) +
                ((node.level+1)*VERTICAL_SPACING));
/*      System.out.println("x=" + ((columns[node.level]*BOX_WIDTH) +
                ((columns[node.level]+1)*HORIZONTAL_SPACING)) + " /y=" +
                ((node.level*BOX_HEIGHT) + ((node.level+1)*VERTICAL_SPACING)));
*/
      if (node.isReceiver()) {
        node.setDimensions(BOX_WIDTH, RECEIVER_BOX_HEIGHT);
        max_height = node.getY() + RECEIVER_BOX_HEIGHT + VERTICAL_SPACING;
        ReceiverNode rn = sessionData.receiverList.findReceiverNode(
                           sessionData.receiverList.getReceiverRoot(), node.IP);
        if (rn != null) rn.setPlacedInTree(true);
      } else {
        node.setDimensions(BOX_WIDTH, BOX_HEIGHT);
        max_height = node.getY() + BOX_HEIGHT + VERTICAL_SPACING;
      }
      max_width = node.getX() + BOX_WIDTH + HORIZONTAL_SPACING;
    
      columns[node.level]++;
//      System.out.println("TreePanel: Setting level " + node.level 
//                         + " and all below to " + columns[node.level]);
      for (int i=node.level-1; i > 0; i--) {
        columns[i] = columns[node.level]; 
      }
   
      if ((node.isLeaf()) || (node.allChildrenPruned())) {
        //Debug.out(debugLevel, Debug.DATA, "TreePanel: Setting above level " + 
        //          node.level + " to " + columns[node.level]);
        for (int i=node.level+1; i < MAX_HOPS; i++) {
          columns[i] = columns[node.level]; 
        }
      }
      node.magicNumber = magicNumber;
    }

    for (int i=0;i<node.childCount();i++) {
      TreeNode child = node.getChild(i);
      if (child.isExpanded())
        child_extent = computeTree(child);

      if (child_extent != null)  {
        if (child_extent.width > max_width) 
          max_width = child_extent.width;  
        if (child_extent.height > max_height) 
          max_height = child_extent.height;  
      }
    }
    return new Dimension(max_width, max_height);
  }
  
  public int computeReceivers(ReceiverNode rn, int x, int y, 
                                    int max_width) {
//    System.out.println("TreePanel: computeReceivers()");

    if (rn != null) {
      if (! rn.getPlacedInTree()) {
        rn.setCoordinates(x,y,BOX_WIDTH,RECEIVER_BOX_HEIGHT);
        x = x + BOX_WIDTH + VERTICAL_SPACING;
      }
      max_width = computeReceivers(rn.getNextNode(), x, y, max_width);
    }
    if (x > max_width)
      return(x);
    else 
      return(max_width);
  }

  public int computeSources(SourceNode sn, int x, int y, int max_width) {
//    System.out.println("TreePanel: computeSources()");

    if (sn != null) {
      sn.setCoordinates(x,y,BOX_WIDTH,BOX_HEIGHT);
      x = x + BOX_WIDTH + VERTICAL_SPACING;

      max_width = computeSources(sn.getNextNode(), x, y, max_width);
    } 

    if (x > max_width)
      return(x);
    else 
      return(max_width);
  }

  public void paint(Graphics g) {
// paint will call drawTree, drawReceivers, and drawSource
    //System.out.println("TreePanel: paint()");
    Dimension d=null;
    boolean print=false;

    if (g instanceof PrintGraphics) {
      print = true;
      d = ((PrintGraphics)g).getPrintJob().getPageDimension();
    } else {
      print = false;
      d = getSize();
    }
    // draws into offscreen Graphics 

    Font f = getFont();
    if (f == null) 
      System.out.println("TreePanel: font is null!");
    setFont(new Font(f.getName(), f.getStyle(), fontSize));

    if ((offscreen == null) || (d.width != offscreensize.width) || (d.height != offscreensize.height)) {
      offscreen = createImage(d.width, d.height);
      offscreensize = d;
      offgraphics = offscreen.getGraphics();
      offgraphics.setFont(getFont());
      offgraphics.setColor(background_color);
      offgraphics.fillRect(0, 0, d.width, d.height);
      
      // scroller stuff :                   
      if (scroller!=null) {
        int h_val = scroller.getHorizontalScrollerVal();
        int v_val = scroller.getVerticalScrollerVal();
        int h_vis = d.width;
        int v_vis = d.height;
        int h_max = tree_size.width;
        int v_max = tree_size.height;
        // System.out.println("Vertical : scroller has value "+v_val);
        // System.out.println("Vertical : visible size is "+v_vis+" - maximal size is "+v_max);
        // System.out.println("Horizontal : scroller has value "+h_val);
        // System.out.println("Horizontal : visible size is "+h_vis+" - maximal size is "+h_max);
        if (h_val>h_max) h_val= h_max;
        if (v_val>v_max) v_val= v_max;
        
        scroller.setHorizontalScrollerValues(h_val,h_vis,0,h_max);
        scroller.setVerticalScrollerValues(v_val,v_vis,0,v_max);
        offgraphics.translate(-h_val,-v_val);
      }    

      offgraphics.setColor(Color.black);

      if (sessionData.tree.getRoot() != null) 
        drawTree(offgraphics,sessionData.tree.getRoot());

      if (! sessionData.receiverList.isEmpty()) 
        drawReceivers(sessionData.receiverList.getReceiverRoot(), offgraphics);
      else
        offgraphics.drawString("No Receivers Identified", HORIZONTAL_SPACING,
                               d.height - VERTICAL_SPACING - BOX_HEIGHT);

      if (! sessionData.sourceList.isEmpty()) 
        drawSources(sessionData.sourceList.getSourceRoot(), offgraphics);
      else 
        offgraphics.drawString("No Source Identified", HORIZONTAL_SPACING,
                               VERTICAL_SPACING*2);

    }
    g.drawImage(offscreen, 0, 0, null);
    offgraphics.dispose();
  }

  public void drawTree(Graphics g, TreeNode node) {
    //System.out.println("TreePanel: drawTree("+node+")");

    Dimension linefrom, lineto;

    node.draw(g);
    
    for (int i=0; i < node.childCount(); i++) {
      TreeNode child = node.getChild(i);

      if (child.isExpanded()) {
        linefrom = node.getLineFrom();
        lineto   = child.getLineTo();

        g.setColor(normal_line_color);

        // draw vertical line from parent to just above child's level
        g.drawLine(linefrom.width, linefrom.height, 
                   linefrom.width, 
                   lineto.height - (VERTICAL_SPACING/2));

        // draw vertical line from child to just above child's level 
        g.drawLine(lineto.width, lineto.height, 
                   lineto.width, 
                   lineto.height - (VERTICAL_SPACING/2));

        // draw horizontal line connecting two prior lines
        g.drawLine(lineto.width, 
                   lineto.height - (VERTICAL_SPACING/2),
                   linefrom.width, 
                   lineto.height - (VERTICAL_SPACING/2));
        // draw sub-tree
        drawTree(g,child);
      }
    }
  }
  
  public void drawReceivers(ReceiverNode rn, Graphics g) {
    //System.out.println("TreePanel: drawReceivers()");

    if (rn != null) {
      if (! rn.getPlacedInTree())
        rn.draw(g);
      drawReceivers(rn.getNextNode(), g);
    }
  }

  public void drawSources(SourceNode sn, Graphics g) {
    //System.out.println("TreePanel: drawSources()");

    if (sn != null) {
      sn.draw(g);
      drawSources(sn.getNextNode(), g);
    }
  }

  public void update(Graphics g) {
    //System.out.println("TreePanel: update()");
    // force redraw :
    offscreen = null;
    paint(g);
  }

  public void createImageDump() {
    GIFEncoder encoder = null;
   
    if (offscreen == null) {
      System.out.println("Image is null!");
      return;
    }

    System.out.println("Capturing Image Now");
    try {
      /* Takes image parameter and encodes it to GIF format right here: */
      /* (Note: This might take some time) */
      encoder = new GIFEncoder(offscreen);
    } catch (AWTException e) {
      System.out.println("ImageDump Failed: possibly too many colors?");
    }
   
    /*Save the image to a file*/
    BufferedOutputStream gifFile  = null;
    String fileName = sessionData.getImageDumpDir() + "mhealth.gif";
    try {
      gifFile =  new BufferedOutputStream(new FileOutputStream(fileName,false));

      encoder.Write(gifFile); /* encoder writes gif image to a file*/
    } catch (Exception e) {
      System.out.println("Error: unable to write image dump to :"+ fileName);
    }
    FilePermission fp = new FilePermission(fileName, "read");

    // Now write an html file to contain this image:
    fileName = sessionData.getImageDumpDir() + "mhealth.html";
    FileWriter fw=null;
    try {
      // open log file
      fw = new FileWriter(fileName,false);
    } catch (IOException ioe) {
      System.out.println("Error: unable to write html file to :"+ fileName);
      return;
    }
    try {
      // output html file 
      fw.write("<META http-equiv=\"refresh\" content=\"15; mhealth.html\">\n");
      fw.write("<META http-equiv=\"pragma\" content=\"no-cache\">\n");
      fw.write("<META http-equiv=\"expires\" content=\"Thu, 1 Apr 1999 " +
                "12:00:00 PST\">\n");
      fw.write("<title>MHealth Image Dump</title>\n");
      fw.write("<body bgcolor=\"#CCCCCC\"><center>\n");
      fw.write("<h1>MHealth Image Dump</h1>\n");
      fw.write("<table border=1 valign=top cellpadding=5>\n");
      fw.write("<tr><td align=right><b>Session Address: </b> ");
      fw.write("<td align=left>" + sessionData.getSessionAddr() + 
               "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;\n");
      fw.write("<td align=right><b>Local Time: </b> ");
      fw.write("<td align=left>" + sessionData.getLocalTime() + "</tr>\n");
      fw.write("<tr><td align=right><b>Source Count: </b> ");
      fw.write("<td align=left>" + sessionData.getSourceCount() +
               "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;\n");
      fw.write("<td align=right><b>Receivers (traced/total): </b>");
      fw.write("<td align=left>" + sessionData.getReceiverCount() + "</tr>\n");
      fw.write("<tr><td align=right><b>Status:</b> ");
      fw.write("<td align=left colspan=3>" + sessionData.getStatusText()+"\n");
      fw.write("</tr>\n");
      fw.write("</table><br>\n");
      fw.write("<img src=mhealth.gif>\n");
      fw.write("<hr>For more info on MHealth see");
      fw.write(" <a href=http://imj.ucsb.edu>http://imj.ucsb.edu</a>\n");
      fw.write("<hr></center></body>\n");
      fw.flush();
      fw.close();
      fp = new FilePermission(fileName, "read");
    } catch (IOException ioe) {
      ioe.printStackTrace();
    }
  }
}
